package mountree

type ErrNo uint8

const (
	ErrMountPointAlreadyExists          ErrNo = iota
	ErrMountPointNotExists              ErrNo = iota
	ErrNoAvailableMountPointForThisPath ErrNo = iota
)

func (e ErrNo) Error() string {
	switch e {
	case ErrMountPointAlreadyExists:
		return "mount point already exists"
	case ErrMountPointNotExists:
		return "mount point not exists"
	case ErrNoAvailableMountPointForThisPath:
		return "no available mount point for this path"
	default:
		return "unknown error"
	}
}

var _ error = (*ErrNo)(nil)

func CheckErrorType(err error, errNo ErrNo) bool {
	if err == nil {
		return false
	}
	switch err.(type) {
	case ErrNo:
		return err.(ErrNo) == errNo
	default:
		return false
	}
}
